@@ -22,7 +22,7 @@ from Products.CMFPlone.tests import dummy
 class TestPortlet(PortletsTestCase):
 
     def afterSetUp(self):
-        self.setRoles(('Manager',))
+        self.setRoles(('Manager', ))
 
     def testPortletTypeRegistered(self):
         portlet = getUtility(IPortletType, name='portlets.Navigation')
@@ -70,6 +70,7 @@ class TestPortlet(PortletsTestCase):
         renderer = getMultiAdapter((context, request, view, manager, assignment), IPortletRenderer)
         self.failUnless(isinstance(renderer, navigation.Renderer))
 
+
 class TestRenderer(PortletsTestCase):
 
     def afterSetUp(self):
@@ -118,7 +119,7 @@ class TestRenderer(PortletsTestCase):
         view = self.renderer(self.portal)
         tree = view.getNavTree()
         self.failUnless(tree)
-        self.failUnless(tree.has_key('children'))
+        self.failUnless('children' in tree)
 
     def testCreateNavTreeCurrentItem(self):
         # With the context set to folder2 it should return a dict with
@@ -192,28 +193,42 @@ class TestRenderer(PortletsTestCase):
         self.failUnless(tree)
         # Ensure that our 'doc21' default page is not in the tree.
         self.assertEqual([c for c in tree['children'][-1]['children']
-                                            if c['item'].getPath()[-5:]=='doc21'],[])
+                                            if c['item'].getPath()[-5:]=='doc21'], [])
 
     def testNavTreeMarksParentMetaTypesNotToQuery(self):
         # Make sure that items whose ids are in the idsNotToList navTree
         # property get no_display set to True
         view = self.renderer(self.portal.folder2.file21)
         tree = view.getNavTree()
-        self.assertEqual(tree['children'][-1]['show_children'],True)
+        self.assertEqual(tree['children'][-1]['show_children'], True)
         ntp=self.portal.portal_properties.navtree_properties
         ntp.manage_changeProperties(parentMetaTypesNotToQuery=['Folder'])
         view = self.renderer(self.portal.folder2.file21)
         tree = view.getNavTree()
-        self.assertEqual(tree['children'][-1]['show_children'],False)
+        self.assertEqual(tree['children'][-1]['show_children'], False)
 
     def testCreateNavTreeWithLink(self):
         view = self.renderer(self.portal)
         tree = view.getNavTree()
         for child in tree['children']:
             if child['portal_type'] != 'Link':
-                self.failIf(child['item'].getRemoteUrl)
+                self.failIf(child['getRemoteUrl'])
             if child['Title'] == 'link1':
-                self.failUnlessEqual(child['item'].getRemoteUrl, 'http://plone.org')
+                self.failUnlessEqual(child['getRemoteUrl'], 'http://plone.org')
+                # as Creator, link1 should not use the remote Url
+                self.failIf(child['useRemoteUrl'])
+        
+        self.portal.link1.setCreators(['some_other_user'])
+        self.portal.link1.reindexObject()
+        view = self.renderer(self.portal)
+        tree = view.getNavTree()
+        for child in tree['children']:
+            if child['portal_type'] != 'Link':
+                self.failIf(child['getRemoteUrl'])
+            if child['Title'] == 'link1':
+                self.failUnlessEqual(child['getRemoteUrl'], 'http://plone.org')
+                # as non-Creator user, link1 should use the remote Url
+                self.failUnless(child['useRemoteUrl'])
 
     def testNonStructuralFolderHidesChildren(self):
         # Make sure NonStructuralFolders act as if parentMetaTypesNotToQuery
@@ -246,10 +261,21 @@ class TestRenderer(PortletsTestCase):
         self.failUnless(tree)
         self.assertEqual(len(tree['children']), 0)
 
+    def testIncludeTopWithoutNavigationRoot(self):
+        self.portal.folder2.invokeFactory('Folder', 'folder21')
+        self.portal.folder2.folder21.invokeFactory('Document', 'doc211')
+        view = self.renderer(self.portal.folder2.folder21,
+            assignment=navigation.Assignment(topLevel=0, root=None, includeTop=True))
+        tree = view.getNavTree()
+        self.failUnless(tree)
+        self.failUnless(view.root_is_portal())
+        self.assertEqual(len(tree['children']), 6)
+
     def testTopLevelWithNavigationRoot(self):
         self.portal.folder2.invokeFactory('Folder', 'folder21')
         self.portal.folder2.folder21.invokeFactory('Document', 'doc211')
-        view = self.renderer(self.portal.folder2.folder21, assignment=navigation.Assignment(topLevel=1, root='/folder2'))
+        view = self.renderer(self.portal.folder2.folder21,
+            assignment=navigation.Assignment(topLevel=1, root='/folder2'))
         tree = view.getNavTree()
         self.failUnless(tree)
         self.assertEqual(len(tree['children']), 1)
@@ -282,7 +308,7 @@ class TestRenderer(PortletsTestCase):
         newObject=self.portal.folder1.restrictedTraverse('portal_factory/' + typeName + '/' + id)
         # Will raise a KeyError unless bug is fixed
         view = self.renderer(newObject, assignment=navigation.Assignment(topLevel=1))
-        tree = view.getNavTree()
+        view.getNavTree()
 
     def testShowAllParentsOverridesBottomLevel(self):
         view = self.renderer(self.portal.folder2.file21, assignment=navigation.Assignment(bottomLevel=1, topLevel=0))
@@ -364,12 +390,12 @@ class TestRenderer(PortletsTestCase):
         factory = self.portal.manage_addProduct['PythonScripts']
         factory.manage_addPythonScript('getCustomNavQuery')
         script = self.portal.getCustomNavQuery
-        script.ZPythonScript_edit('','return {"review_state":"published"}')
-        self.assertEqual(self.portal.getCustomNavQuery(),{"review_state":"published"})
+        script.ZPythonScript_edit('', 'return {"review_state": "published"}')
+        self.assertEqual(self.portal.getCustomNavQuery(), {"review_state": "published"})
         view = self.renderer(self.portal.folder2)
         tree = view.getNavTree()
         self.failUnless(tree)
-        self.failUnless(tree.has_key('children'))
+        self.failUnless('children' in tree)
         #Should only contain current object
         self.assertEqual(len(tree['children']), 1)
         #change workflow for folder1
@@ -390,7 +416,7 @@ class TestRenderer(PortletsTestCase):
         view = self.renderer(self.portal.folder2)
         tree = view.getNavTree()
         self.failUnless(tree)
-        self.failUnless(tree.has_key('children'))
+        self.failUnless('children' in tree)
         #Should only contain current object
         self.assertEqual(len(tree['children']), 1)
         #change workflow for folder1
@@ -425,10 +451,10 @@ class TestRenderer(PortletsTestCase):
         self.assertEqual(tree['children'][0]['item'].getPath(), '/plone/folder1/doc11')
 
     def testIsCurrentParentWithOverlapingNames(self):
-        self.setRoles(['Manager',])
+        self.setRoles(['Manager', ])
         self.portal.invokeFactory('Folder', 'folder2x')
         self.portal.folder2x.invokeFactory('Document', 'doc2x1')
-        self.setRoles(['Member',])
+        self.setRoles(['Member', ])
         view = self.renderer(self.portal.folder2x.doc2x1)
         tree = view.getNavTree()
         self.failUnless(tree)
@@ -439,15 +465,58 @@ class TestRenderer(PortletsTestCase):
         folder2_node = [n for n in tree['children'] if n['path'] == '/plone/folder2'][0]
         self.failIf(folder2_node['currentParent'])
 
-    def testINavigationRootAvailability(self):
+    def testPortletNotDisplayedOnINavigationRoot(self):
+        """test that navigation portlet does not show on INavigationRoot
+        folder
+        """
         self.failIf(INavigationRoot.providedBy(self.portal.folder1))
-        self.portal.folder1.invokeFactory('Folder', 'folder1_1')
+
+        # make folder1 as navigation root
         directlyProvides(self.portal.folder1, INavigationRoot)
         self.failUnless(INavigationRoot.providedBy(self.portal.folder1))
-        view = self.renderer(self.portal.folder1, assignment=navigation.Assignment(bottomLevel=0, topLevel=1, root=None))
-        tree = view.getNavTree()
-        root = view.getNavRoot()
-        self.failIf(root is not None and len(tree['children']) > 0)
+
+        # add nested subfolder in folder1
+        self.portal.folder1.invokeFactory('Folder', 'folder1_1')
+
+        # make a navigation portlet
+        assignment = navigation.Assignment(bottomLevel=0, topLevel=1,
+                root=None)
+        portlet = self.renderer(self.portal.folder1, assignment=assignment)
+
+        # check there is no portlet
+        self.failIf(portlet.available)
+
+    def testINavigationRootWithRelativeRootSet(self):
+        """test that navigation portlet uses relative root set by user
+           even in INavigationRoot case.
+        """
+        self.failIf(INavigationRoot.providedBy(self.portal.folder1))
+
+        # make folder1 as navigation root
+        directlyProvides(self.portal.folder1, INavigationRoot)
+        self.failUnless(INavigationRoot.providedBy(self.portal.folder1))
+
+        # add two nested subfolders in folder1
+        self.portal.folder1.invokeFactory('Folder', 'folder1_1')
+        self.portal.folder1.folder1_1.invokeFactory('Folder', 'folder1_1_1')
+
+        # make a navigation portlet with navigation root set
+        assignment = navigation.Assignment(bottomLevel=0, topLevel=0,
+                root='/folder1/folder1_1')
+        portlet = self.renderer(self.portal.folder1.folder1_1,
+                assignment=assignment)
+
+        # check there is a portlet
+        self.failUnless(portlet.available)
+
+        # check that portlet root is actually the one specified
+        root = portlet.getNavRoot()
+        self.assertEqual(root.getId(), 'folder1_1')
+
+        # check that portlet tree actually includes children
+        tree = portlet.getNavTree()
+        self.assertEqual(len(tree['children']), 1)
+        self.assertEqual(tree['children'][0]['item'].getPath(), '/plone/folder1/folder1_1/folder1_1_1')
 
     def testImgConditionalOnTypeIcon(self):
         """The <img> element should not render if the content type has
@@ -459,13 +528,13 @@ class TestRenderer(PortletsTestCase):
         view = self.renderer(self.portal)
         tree = view.getNavTree()
         self.failIf(tree['children'][3]['item_icon'].html_tag())
-        
+
     def testPortletsTitle(self):
-        """If portlet's name is not explicitely specified we show 
+        """If portlet's name is not explicitely specified we show
            default fallback 'Navigation', translate it and hide it
            with CSS."""
         view = self.renderer(self.portal)
-        tree = view.getNavTree()
+        view.getNavTree()
         self.assertEqual(view.title(), "Navigation")
         self.failIf(view.hasName())
         view.data.name = 'New navigation title'
